@@ -34,7 +34,9 @@ from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 from nuitka.PythonFlavors import (
     isAnacondaPython,
     isApplePython,
+    isCPythonOfficialPackage,
     isDebianPackagePython,
+    isHomebrewPython,
     isMSYS2MingwPython,
     isNuitkaPython,
     isPyenvPython,
@@ -45,8 +47,10 @@ from nuitka.utils.Utils import (
     getArchitecture,
     getLinuxDistribution,
     getOS,
+    getWindowsRelease,
     isLinux,
     isPosixWindows,
+    withNoSyntaxWarning,
 )
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
@@ -71,6 +75,8 @@ def _getPythonFlavor():
         return "WinPython"
     elif isDebianPackagePython():
         return "Debian Python"
+    elif isHomebrewPython():
+        return "Homebrew Python"
     elif isApplePython():
         return "Apple Python"
     elif isPyenvPython():
@@ -79,6 +85,8 @@ def _getPythonFlavor():
         return "MSYS2 Posix"
     elif isMSYS2MingwPython():
         return "MSYS2 MinGW"
+    elif isCPythonOfficialPackage():
+        return "CPython Official"
     else:
         return "Unknown"
 
@@ -95,19 +103,47 @@ def _getVersionInformationValues():
     yield "Arch: %s" % getArchitecture()
 
     if isLinux():
-        yield "Distribution: %s %s" % getLinuxDistribution()
+        dist_name, dist_base, dist_version = getLinuxDistribution()
+
+        if dist_base is not None:
+            yield "Distribution: %s (based on %s) %s" % (
+                dist_name,
+                dist_base,
+                dist_version,
+            )
+        else:
+            yield "Distribution: %s %s" % (dist_name, dist_version)
+
+    if getOS() == "Windows":
+        yield "WindowsRelease: %s" % getWindowsRelease()
 
 
-parser = OptionParser(
+class OurOptionParser(OptionParser):
+    # spell-checker: ignore rargs
+    def _process_long_opt(self, rargs, values):
+        arg = rargs[0]
+
+        if "=" not in arg:
+            opt = self._match_long_opt(arg)
+            option = self._long_opt[opt]
+            if option.takes_value():
+                self.error(
+                    "The '%s' option requires an argument with '%s='." % (opt, opt)
+                )
+
+        return OptionParser._process_long_opt(self, rargs, values)
+
+
+parser = OurOptionParser(
     usage=usage,
     version="\n".join(_getVersionInformationValues()),
 )
 
 parser.add_option(
     "--module",
-    action="store_false",
-    dest="executable",
-    default=True,
+    action="store_true",
+    dest="module_mode",
+    default=False,
     help="""\
 Create an extension module executable instead of a program. Defaults to off.""",
 )
@@ -120,9 +156,8 @@ parser.add_option(
     help="""\
 Enable standalone mode for output. This allows you to transfer the created binary
 to other machines without it using an existing Python installation. This also
-means it will become big. It implies these option: "--follow-imports". You may also
-want to use "--python-flag=no_site" to avoid the "site.py" module, which can save
-a lot of code dependencies. Defaults to off.""",
+means it will become big. It implies these option: "--follow-imports" and
+"--python-flag=no_site". Defaults to off.""",
 )
 
 parser.add_option(
@@ -152,20 +187,6 @@ parser.add_option(
     help=SUPPRESS_HELP,
 )
 
-
-if os.name == "nt":
-    parser.add_option(
-        "--python-arch",
-        action="store",
-        dest="python_arch",
-        choices=("x86", "x86_64"),
-        default=None,
-        help="""\
-Architecture of Python to use. One of "x86" or "x86_64".
-Defaults to what you run Nuitka with (currently "%s")."""
-        % (getArchitecture()),
-    )
-
 parser.add_option(
     "--python-debug",
     action="store_true",
@@ -188,7 +209,7 @@ enforces a specific mode. These are options that also exist to standard
 Python executable. Currently supported: "-S" (alias "no_site"),
 "static_hashes" (do not use hash randomization), "no_warnings" (do not
 give Python runtime warnings), "-O" (alias "no_asserts"), "no_docstrings"
-(do not use docstrings). Default empty.""",
+(do not use docstrings), "-u" (alias "unbuffered") and "-m".  Default empty.""",
 )
 
 parser.add_option(
@@ -228,7 +249,8 @@ parser.add_option(
     default=False,
     help="""\
 Allow Nuitka to download external code if necessary, e.g. dependency
-walker, ccache, and even gcc on Windows.""",
+walker, ccache, and even gcc on Windows. To disable, redirect input
+from nul device, e.g. "</dev/null" or "<NUL:". Default is to prompt.""",
 )
 
 
@@ -488,6 +510,16 @@ parser.add_option_group(dump_group)
 
 codegen_group = OptionGroup(parser, "Code generation choices")
 
+codegen_group.add_option(
+    "--disable-bytecode-cache",
+    action="store_true",
+    dest="disable_bytecode_cache",
+    default=False,
+    help="""\
+Do not reuse dependency analysis results for modules, esp. from standard library,
+that are included as bytecode.""",
+)
+
 codegen_group.add_option(
     "--full-compat",
     action="store_false",
@@ -609,12 +641,13 @@ Enable vmprof based profiling of time spent. Not working currently. Defaults to
 )
 
 debug_group.add_option(
-    "--graph",
+    "--internal-graph",
     action="store_true",
     dest="graph",
     default=False,
     help="""\
-Create graph of optimization process. Defaults to off.""",
+Create graph of optimization process internals, do not use for whole programs, but only
+for small test cases. Defaults to off.""",
 )
 
 debug_group.add_option(
@@ -749,11 +782,11 @@ c_compiler_group.add_option(
     default=None,
     help="""\
 Enforce the use of specific MSVC version on Windows. Allowed values
-are e.g. "14.2" (MSVC 2019), specify an illegal value for a list of
-installed compilers, or use "latest". Notice that only latest
-MSVC is really supported, and you can use "latest" to enforce that.
+are e.g. "14.3" (MSVC 2022) and other MSVC version numbers, specify
+"list" for a list of installed compilers, or use "latest".
 
-Defaults to MSVC on Windows being used if installed, otherwise MinGW64.""",
+Defaults to latest MSVC being used if installed, otherwise MinGW64
+is used.""",
 )
 
 c_compiler_group.add_option(
@@ -937,6 +970,15 @@ tracing_group.add_option(
 Where to output --show-modules, should be a filename. Default is standard output.""",
 )
 
+tracing_group.add_option(
+    "--report",
+    action="store",
+    dest="compilation_report_filename",
+    default=None,
+    help="""\
+Report module inclusion in an XML output file. Default is off.""",
+)
+
 tracing_group.add_option(
     "--verbose",
     action="store_true",
@@ -1105,6 +1147,7 @@ windows_group.add_option(
 
 windows_group.add_option(
     "--windows-onefile-tempdir-spec",
+    "--onefile-tempdir-spec",
     action="store",
     dest="onefile_tempdir_spec",
     metavar="ONEFILE_TEMPDIR_SPEC",
@@ -1143,16 +1186,21 @@ parser.add_option_group(windows_group)
 macos_group = OptionGroup(parser, "macOS specific controls")
 
 macos_group.add_option(
-    "--macos-onefile-icon",
-    action="append",
-    dest="icon_path",
-    metavar="ICON_PATH",
-    default=[],
-    help="Add executable icon for binary to use. Can be given only one time. Defaults to Python icon if available.",
+    "--macos-target-arch",
+    action="store",
+    dest="macos_target_arch",
+    choices=("universal", "arm64", "x86_64"),
+    metavar="MACOS_TARGET_ARCH",
+    default=None,
+    help="""\
+What architectures is this to supposed to run on. Default and limit
+is what the running Python allows for. Default is "native" which is
+the architecture the Python is run with.""",
 )
 
 macos_group.add_option(
     "--macos-disable-console",
+    "--disable-console",
     action="store_true",
     dest="disable_console",
     default=False,
@@ -1172,6 +1220,16 @@ application. Currently experimental and incomplete. Currently this
 is the only way to unlock disabling of console.Defaults to off.""",
 )
 
+macos_group.add_option(
+    "--macos-onefile-icon",
+    action="append",
+    dest="icon_path",
+    metavar="ICON_PATH",
+    default=[],
+    help="Add executable icon for binary to use. Can be given only one time. Defaults to Python icon if available.",
+)
+
+
 macos_group.add_option(
     "--macos-signed-app-name",
     action="store",
@@ -1220,6 +1278,16 @@ linux_group.add_option(
     help="Add executable icon for onefile binary to use. Can be given only one time. Defaults to Python icon if available.",
 )
 
+linux_group.add_option(
+    "--linux-onefile-compression",
+    action="store",
+    dest="appimage_compression",
+    choices=("gzip", "xz"),
+    metavar="COMPRESSION",
+    default="gzip",
+    help="Compression method to use for Linux onefile builds. Defaults to gzip for faster decompression",
+)
+
 parser.add_option_group(linux_group)
 
 plugin_group = OptionGroup(parser, "Plugin control")
@@ -1299,9 +1367,12 @@ def _considerPluginOptions(logger):
     # Cyclic dependency on plugins during parsing of command line.
     from nuitka.plugins.Plugins import (
         addPluginCommandLineOptions,
+        addStandardPluginCommandlineOptions,
         addUserPluginCommandLineOptions,
     )
 
+    addStandardPluginCommandlineOptions(parser=parser, data_files_tags=data_files_tags)
+
     for arg in sys.argv[1:]:
         if arg.startswith(("--enable-plugin=", "--plugin-enable=")):
             plugin_names = arg[16:]
@@ -1346,13 +1417,17 @@ def _expandProjectArg(arg, filename_arg, for_eval):
         "MAIN_DIRECTORY": wrap(os.path.dirname(filename_arg) or "."),
     }
 
-    if getOS() != "Linux":
-        dist_info = "N/A", "0"
-    else:
+    if isLinux():
         dist_info = getLinuxDistribution()
+    else:
+        dist_info = "N/A", "N/A", "0"
 
     values["Linux_Distribution_Name"] = dist_info[0]
-    values["Linux_Distribution_Version"] = dist_info[1]
+    values["Linux_Distribution_Base"] = dist_info[1] or dist_info[0]
+    values["Linux_Distribution_Version"] = dist_info[2]
+
+    if getOS() == "Windows":
+        values["WindowsRelease"] = getWindowsRelease()
 
     arg = arg.format(**values)
 
@@ -1361,9 +1436,6 @@ def _expandProjectArg(arg, filename_arg, for_eval):
 
 def _getProjectOptions(logger, filename_arg, module_mode):
     # Complex stuff, pylint: disable=too-many-branches,too-many-locals
-    # Do it only once.
-    if os.environ.get("NUITKA_REEXECUTION", "0") == "1":
-        return
 
     if os.path.isdir(filename_arg):
         if module_mode:
@@ -1422,9 +1494,11 @@ def _getProjectOptions(logger, filename_arg, module_mode):
                 arg = arg[:-1]
 
                 expanded = _expandProjectArg(arg, filename_arg, for_eval=True)
-                r = eval(  # We allow the user to run any code, pylint: disable=eval-used
-                    expanded
-                )
+
+                with withNoSyntaxWarning():
+                    r = eval(  # We allow the user to run any code, pylint: disable=eval-used
+                        expanded
+                    )
 
                 # Likely mistakes, e.g. with "in" tests.
                 if r is not True and r is not False:
