@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,14 +18,12 @@
 """ Eval/exec/execfile/compile built-in related codes. """
 
 from nuitka import Options
-from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
 from nuitka.PythonVersions import python_version
 
 from .CodeHelpers import (
     generateExpressionCode,
     withObjectCodeTemporaryAssignment,
 )
-from .ConstantCodes import getConstantAccess
 from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode, getReleaseCode
 from .PythonAPICodes import getReferenceExportCode
 from .VariableCodes import getVariableAssignmentCode
@@ -85,50 +83,53 @@ def generateBuiltinCompileCode(to_name, expression, emit, context):
 
     generateExpressionCode(
         to_name=source_name,
-        expression=expression.getSourceCode(),
+        expression=expression.subnode_source,
         emit=emit,
         context=context,
     )
     generateExpressionCode(
         to_name=filename_name,
-        expression=expression.getFilename(),
+        expression=expression.subnode_filename,
         emit=emit,
         context=context,
     )
     generateExpressionCode(
-        to_name=mode_name, expression=expression.getMode(), emit=emit, context=context
+        to_name=mode_name,
+        expression=expression.subnode_mode,
+        emit=emit,
+        context=context,
     )
 
-    if expression.getFlags() is not None:
+    if expression.subnode_flags is not None:
         flags_name = context.allocateTempName("compile_flags")
 
         generateExpressionCode(
             to_name=flags_name,
-            expression=expression.getFlags(),
+            expression=expression.subnode_flags,
             emit=emit,
             context=context,
         )
     else:
         flags_name = "NULL"
 
-    if expression.getDontInherit() is not None:
+    if expression.subnode_dont_inherit is not None:
         dont_inherit_name = context.allocateTempName("compile_dont_inherit")
 
         generateExpressionCode(
             to_name=dont_inherit_name,
-            expression=expression.getDontInherit(),
+            expression=expression.subnode_dont_inherit,
             emit=emit,
             context=context,
         )
     else:
         dont_inherit_name = "NULL"
 
-    if expression.getOptimize() is not None:
-        optimize_name = context.allocateTempName("compile_dont_inherit")
+    if expression.subnode_optimize is not None:
+        optimize_name = context.allocateTempName("compile_optimize")
 
         generateExpressionCode(
             to_name=optimize_name,
-            expression=expression.getOptimize(),
+            expression=expression.subnode_optimize,
             emit=emit,
             context=context,
         )
@@ -165,7 +166,7 @@ def _getBuiltinCompileCode(
     emit,
     context,
 ):
-    if python_version < 300:
+    if python_version < 0x300:
         args = (source_name, filename_name, mode_name, flags_name, dont_inherit_name)
     else:
         args = (
@@ -236,9 +237,9 @@ def getBuiltinEvalCode(
 
 
 def generateExecCode(statement, emit, context):
-    source_arg = statement.getSourceCode()
-    globals_arg = statement.getGlobals()
-    locals_arg = statement.getLocals()
+    source_arg = statement.subnode_source
+    globals_arg = statement.subnode_globals_arg
+    locals_arg = statement.subnode_locals_arg
 
     source_name = context.allocateTempName("exec_source")
     globals_name = context.allocateTempName("exec_globals")
@@ -261,11 +262,12 @@ def generateExecCode(statement, emit, context):
     filename_name = context.allocateTempName("exec_filename")
 
     # Default filename with origin in improved mode.
-    getConstantAccess(
+    filename_name.getCType().emitAssignmentCodeFromConstant(
         to_name=filename_name,
         constant="<string>"
         if Options.is_fullcompat
         else "<string at %s>" % source_ref.getAsString(),
+        may_escape=False,
         emit=emit,
         context=context,
     )
@@ -276,61 +278,61 @@ def generateExecCode(statement, emit, context):
     getReferenceExportCode(source_name, emit, context)
     context.addCleanupTempName(source_name)
 
-    old_source_ref = context.setCurrentSourceCodeReference(
+    with context.withCurrentSourceCodeReference(
         locals_arg.getSourceReference()
         if Options.is_fullcompat
         else statement.getSourceReference()
-    )
+    ):
 
-    res_name = context.getBoolResName()
+        res_name = context.getBoolResName()
 
-    emit(
-        "%s = EXEC_FILE_ARG_HANDLING(&%s, &%s);"
-        % (res_name, source_name, filename_name)
-    )
+        emit(
+            "%s = EXEC_FILE_ARG_HANDLING(&%s, &%s);"
+            % (res_name, source_name, filename_name)
+        )
 
-    getErrorExitBoolCode(condition="%s == false" % res_name, emit=emit, context=context)
+        getErrorExitBoolCode(
+            condition="%s == false" % res_name, emit=emit, context=context
+        )
 
-    compiled_name = context.allocateTempName("exec_compiled")
+        compiled_name = context.allocateTempName("exec_compiled")
 
-    _getBuiltinCompileCode(
-        to_name=compiled_name,
-        source_name=source_name,
-        filename_name=filename_name,
-        mode_name=context.getConstantCode(constant="exec"),
-        flags_name="NULL",
-        dont_inherit_name="NULL",
-        optimize_name="NULL",
-        emit=emit,
-        context=context,
-    )
+        _getBuiltinCompileCode(
+            to_name=compiled_name,
+            source_name=source_name,
+            filename_name=filename_name,
+            mode_name=context.getConstantCode(constant="exec"),
+            flags_name="NULL",
+            dont_inherit_name="NULL",
+            optimize_name="NULL",
+            emit=emit,
+            context=context,
+        )
 
-    to_name = context.allocateTempName("exec_result")
+        to_name = context.allocateTempName("exec_result")
 
-    emit(
-        "%s = EVAL_CODE(%s, %s, %s);"
-        % (to_name, compiled_name, globals_name, locals_name)
-    )
+        emit(
+            "%s = EVAL_CODE(%s, %s, %s);"
+            % (to_name, compiled_name, globals_name, locals_name)
+        )
 
-    getErrorExitCode(
-        check_name=to_name,
-        release_names=(
-            compiled_name,
-            globals_name,
-            locals_name,
-            source_name,
-            filename_name,
-        ),
-        emit=emit,
-        context=context,
-    )
+        getErrorExitCode(
+            check_name=to_name,
+            release_names=(
+                compiled_name,
+                globals_name,
+                locals_name,
+                source_name,
+                filename_name,
+            ),
+            emit=emit,
+            context=context,
+        )
 
-    # Immediately release the exec result, no point in keeping it, it's a
-    # statement.
-    context.addCleanupTempName(to_name)
-    getReleaseCode(release_name=to_name, emit=emit, context=context)
-
-    context.setCurrentSourceCodeReference(old_source_ref)
+        # Immediately release the exec result, no point in keeping it, it's a
+        # statement.
+        context.addCleanupTempName(to_name)
+        getReleaseCode(release_name=to_name, emit=emit, context=context)
 
 
 def _generateEvalCode(to_name, node, emit, context):
@@ -339,19 +341,25 @@ def _generateEvalCode(to_name, node, emit, context):
     locals_name = context.allocateTempName("eval_locals")
 
     generateExpressionCode(
-        to_name=source_name, expression=node.getSourceCode(), emit=emit, context=context
+        to_name=source_name, expression=node.subnode_source, emit=emit, context=context
     )
 
     generateExpressionCode(
-        to_name=globals_name, expression=node.getGlobals(), emit=emit, context=context
+        to_name=globals_name,
+        expression=node.subnode_globals_arg,
+        emit=emit,
+        context=context,
     )
 
     generateExpressionCode(
-        to_name=locals_name, expression=node.getLocals(), emit=emit, context=context
+        to_name=locals_name,
+        expression=node.subnode_locals_arg,
+        emit=emit,
+        context=context,
     )
 
     if node.isExpressionBuiltinEval() or (
-        python_version >= 300 and node.isExpressionBuiltinExec()
+        python_version >= 0x300 and node.isExpressionBuiltinExec()
     ):
         filename = "<string>"
     else:
@@ -381,7 +389,7 @@ def generateEvalCode(to_name, expression, emit, context):
 
 
 def generateExecfileCode(to_name, expression, emit, context):
-    assert python_version < 300
+    assert python_version < 0x300
 
     with withObjectCodeTemporaryAssignment(
         to_name, "execfile_result", expression, emit, context
@@ -392,23 +400,18 @@ def generateExecfileCode(to_name, expression, emit, context):
 
 
 def generateLocalsDictSyncCode(statement, emit, context):
-    locals_arg = statement.getLocals()
+    locals_arg = statement.subnode_locals_arg
     locals_name = context.allocateTempName("sync_locals")
 
     generateExpressionCode(
         to_name=locals_name, expression=locals_arg, emit=emit, context=context
     )
 
-    old_source_ref = context.setCurrentSourceCodeReference(
-        statement.getSourceReference()
-    )
-
-    _getStoreLocalsCode(
-        locals_name=locals_name,
-        variable_traces=statement.getPreviousVariablesTraces(),
-        is_dict=locals_arg.getTypeShape() is tshape_dict,
-        emit=emit,
-        context=context,
-    )
-
-    context.setCurrentSourceCodeReference(old_source_ref)
+    with context.withCurrentSourceCodeReference(statement.getSourceReference()):
+        _getStoreLocalsCode(
+            locals_name=locals_name,
+            variable_traces=statement.getPreviousVariablesTraces(),
+            is_dict=locals_arg.hasShapeDictionaryExact(),
+            emit=emit,
+            context=context,
+        )
