@@ -23,24 +23,29 @@ import glob
 import os
 
 from nuitka import ModuleRegistry, Options
+from nuitka.Errors import NuitkaForbiddenImportEncounter
 from nuitka.importing import ImportCache, Importing, StandardLibrary
+from nuitka.ModuleRegistry import addUsedModule, getRootTopModule
+from nuitka.pgo.PGO import decideInclusionFromPGO
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import recursion_logger
-from nuitka.utils.FileOperations import listDir, relpath
+from nuitka.utils.FileOperations import listDir
 from nuitka.utils.ModuleNames import ModuleName
 
+from .Importing import getModuleNameAndKindFromFilename, locateModule
 
-def _recurseTo(module_package, module_filename, module_kind):
+
+def _recurseTo(module_name, module_filename, module_kind):
     from nuitka.tree import Building
 
     module, is_added = Building.buildModule(
         module_filename=module_filename,
-        module_package=module_package,
+        module_name=module_name,
         source_code=None,
         is_top=False,
         is_main=False,
-        is_shlib=module_kind == "shlib",
+        is_extension=module_kind == "extension",
         is_fake=False,
         hide_syntax_error=True,
     )
@@ -50,20 +55,17 @@ def _recurseTo(module_package, module_filename, module_kind):
     return module, is_added
 
 
-def recurseTo(
-    signal_change, module_package, module_filename, module_relpath, module_kind, reason
-):
-    if ImportCache.isImportedModuleByPath(module_relpath):
-        try:
-            module = ImportCache.getImportedModuleByPath(module_relpath, module_package)
-        except KeyError:
-            module = None
-    else:
+def recurseTo(signal_change, module_name, module_filename, module_kind, reason):
+    try:
+        module = ImportCache.getImportedModuleByNameAndPath(
+            module_name, module_filename
+        )
+    except KeyError:
         module = None
 
     if module is None:
         module, added_flag = _recurseTo(
-            module_package=module_package,
+            module_name=module_name,
             module_filename=module_filename,
             module_kind=module_kind,
         )
@@ -76,49 +78,87 @@ def recurseTo(
 
 def decideRecursion(module_filename, module_name, module_kind, extra_recursion=False):
     # Many branches, which make decisions immediately, by returning
-    # pylint: disable=too-many-return-statements
+    # pylint: disable=too-many-branches,too-many-return-statements
     if module_name == "__main__":
-        return False, "Main program is not recursed to again."
+        return False, "Main program is not followed to a second time."
+
+    # In -m mode, when including the package, do not duplicate main program.
+    if (
+        Options.hasPythonFlagPackageMode()
+        and not Options.shallMakeModule()
+        and module_name.getBasename() == "__main__"
+    ):
+        if module_name.getPackageName() == getRootTopModule().getRuntimePackageValue():
+            return False, "Main program is already included in package mode."
 
     plugin_decision = Plugins.onModuleEncounter(
-        module_filename, module_name, module_kind
+        module_filename=module_filename,
+        module_name=module_name,
+        module_kind=module_kind,
     )
 
     if plugin_decision is not None:
         return plugin_decision
 
-    if module_kind == "shlib":
+    if module_kind == "extension":
         if Options.isStandaloneMode():
             return True, "Extension module needed for standalone mode."
         else:
-            return False, "Shared library cannot be inspected."
+            return False, "Extension module cannot be inspected."
+
+    # PGO decisions are not overruling plugins, but all command line options, they are
+    # supposed to be applied already.
+    is_stdlib = StandardLibrary.isStandardLibraryPath(module_filename)
+
+    if not is_stdlib or Options.shallFollowStandardLibrary():
+        # TODO: Bad placement of this function or should PGO also know about
+        # bytecode modules loaded or not.
+        from nuitka.tree.Building import decideCompilationMode
+
+        if (
+            decideCompilationMode(is_top=False, module_name=module_name, for_pgo=True)
+            == "compiled"
+        ):
+            pgo_decision = decideInclusionFromPGO(
+                module_name=module_name,
+                module_kind=module_kind,
+            )
+
+            if pgo_decision is not None:
+                return pgo_decision, "PGO based decision"
 
     no_case, reason = module_name.matchesToShellPatterns(
         patterns=Options.getShallFollowInNoCase()
     )
 
     if no_case:
-        return (False, "Module %s instructed by user to not recurse to." % reason)
+        return (False, "Module %s instructed by user to not follow to." % reason)
 
     any_case, reason = module_name.matchesToShellPatterns(
         patterns=Options.getShallFollowModules()
     )
 
     if any_case:
-        return (True, "Module %s instructed by user to recurse to." % reason)
+        return (True, "Module %s instructed by user to follow to." % reason)
 
     if Options.shallFollowNoImports():
-        return (False, "Requested to not recurse at all.")
+        return (False, "Instructed by user to not follow at all.")
 
-    if StandardLibrary.isStandardLibraryPath(module_filename):
-        return (
-            Options.shallFollowStandardLibrary(),
-            "Requested to %srecurse to standard library."
-            % ("" if Options.shallFollowStandardLibrary() else "not "),
-        )
+    if is_stdlib and Options.shallFollowStandardLibrary():
+        return (True, "Instructed by user to follow to standard library.")
 
     if Options.shallFollowAllImports():
-        return (True, "Requested to recurse to all non-standard library modules.")
+        if is_stdlib:
+            if StandardLibrary.isStandardLibraryNoAutoInclusionModule(module_name):
+                return (
+                    True,
+                    "Instructed by user to follow all modules, including non-automatic standard library modules.",
+                )
+        else:
+            return (
+                True,
+                "Instructed by user to follow to all non-standard library modules.",
+            )
 
     # Means, we were not given instructions how to handle things.
     if extra_recursion:
@@ -137,19 +177,16 @@ def considerFilename(module_filename):
         module_filename = os.path.abspath(module_filename)
 
         module_name = os.path.basename(module_filename)
-        module_relpath = relpath(module_filename)
 
-        return module_filename, module_relpath, module_name
+        return module_filename, module_name
     elif module_filename.endswith(".py"):
         module_name = os.path.basename(module_filename)[:-3]
-        module_relpath = relpath(module_filename)
 
-        return module_filename, module_relpath, module_name
+        return module_filename, module_name
     elif module_filename.endswith(".pyw"):
         module_name = os.path.basename(module_filename)[:-4]
-        module_relpath = relpath(module_filename)
 
-        return module_filename, module_relpath, module_name
+        return module_filename, module_name
     else:
         return None
 
@@ -166,6 +203,9 @@ def isSameModulePath(path1, path2):
 def checkPluginSinglePath(plugin_filename, module_package):
     # Many branches, for the decision is very complex, pylint: disable=too-many-branches
 
+    # The importing wants these to be unique.
+    plugin_filename = os.path.abspath(plugin_filename)
+
     if Options.isShowInclusion():
         recursion_logger.info(
             "Checking detail plug-in path '%s' '%s':"
@@ -178,6 +218,12 @@ def checkPluginSinglePath(plugin_filename, module_package):
 
     module_name = ModuleName.makeModuleNameInPackage(module_name, module_package)
 
+    if module_kind == "extension" and not Options.isStandaloneMode():
+        recursion_logger.warning(
+            "Cannot include '%s' unless using at least standalone mode."
+            % module_name.asString()
+        )
+
     if module_kind is not None:
         decision, reason = decideRecursion(
             module_filename=plugin_filename,
@@ -187,13 +233,10 @@ def checkPluginSinglePath(plugin_filename, module_package):
         )
 
         if decision:
-            module_relpath = relpath(plugin_filename)
-
             module = recurseTo(
                 signal_change=None,
                 module_filename=plugin_filename,
-                module_relpath=module_relpath,
-                module_package=module_package,
+                module_name=module_name,
                 module_kind=module_kind,
                 reason=reason,
             )
@@ -249,7 +292,7 @@ def checkPluginSinglePath(plugin_filename, module_package):
 
                 elif module.isCompiledPythonModule():
                     ModuleRegistry.addRootModule(module)
-                elif module.isPythonShlibModule():
+                elif module.isPythonExtensionModule():
                     if Options.isStandaloneMode():
                         ModuleRegistry.addRootModule(module)
 
@@ -260,8 +303,6 @@ def checkPluginSinglePath(plugin_filename, module_package):
 
 
 def checkPluginPath(plugin_filename, module_package):
-    plugin_filename = os.path.normpath(plugin_filename)
-
     if Options.isShowInclusion():
         recursion_logger.info(
             "Checking top level plug-in path %s %s" % (plugin_filename, module_package)
@@ -307,3 +348,101 @@ def checkPluginFilenamePattern(pattern):
 
     if not found:
         recursion_logger.warning("Didn't match any files against pattern %r." % pattern)
+
+
+def _addParentPackageUsages(using_module, module_name, signal_change, source_ref):
+    for parent_package_name in module_name.getParentPackageNames():
+        _parent_package_name, parent_package_filename, _finding = locateModule(
+            module_name=parent_package_name, parent_package=None, level=0
+        )
+
+        assert parent_package_filename is not None, parent_package_name
+        assert _parent_package_name == parent_package_name
+
+        _parent_package_name, package_module_kind = getModuleNameAndKindFromFilename(
+            parent_package_filename
+        )
+
+        _decision, reason = decideRecursion(
+            module_filename=parent_package_filename,
+            module_name=parent_package_name,
+            module_kind=package_module_kind,
+        )
+
+        used_package_module = recurseTo(
+            signal_change=signal_change,
+            module_name=parent_package_name,
+            module_filename=parent_package_filename,
+            module_kind=package_module_kind,
+            reason=reason,
+        )
+
+        addUsedModule(
+            module=used_package_module,
+            using_module=using_module,
+            usage_tag="package",
+            reason=reason,
+            source_ref=source_ref,
+        )
+
+
+def considerUsedModules(module, signal_change):
+    for (
+        used_module_name,
+        used_module_filename,
+        finding,
+        level,
+        source_ref,
+    ) in module.getUsedModules():
+        if finding == "not-found":
+            Importing.warnAbout(
+                importing=module,
+                source_ref=source_ref,
+                module_name=used_module_name,
+                level=level,
+            )
+
+        try:
+            if used_module_filename is None:
+                continue
+
+            _module_name, module_kind = getModuleNameAndKindFromFilename(
+                used_module_filename
+            )
+
+            decision, reason = decideRecursion(
+                module_filename=used_module_filename,
+                module_name=used_module_name,
+                module_kind=module_kind,
+            )
+
+            if decision:
+                _addParentPackageUsages(
+                    using_module=module,
+                    module_name=used_module_name,
+                    signal_change=signal_change,
+                    source_ref=source_ref,
+                )
+
+                used_module = recurseTo(
+                    signal_change=signal_change,
+                    module_name=used_module_name,
+                    module_filename=used_module_filename,
+                    module_kind=module_kind,
+                    reason=reason,
+                )
+
+                addUsedModule(
+                    module=used_module,
+                    using_module=module,
+                    usage_tag="import",
+                    reason=reason,
+                    source_ref=source_ref,
+                )
+        except NuitkaForbiddenImportEncounter as e:
+            recursion_logger.sysexit(
+                "Error, forbidden import of '%s' in module '%s' at '%s' encountered."
+                % (e, module.getFullName().asString(), source_ref.getAsString())
+            )
+
+    Plugins.considerImplicitImports(module=module, signal_change=signal_change)
