@@ -26,23 +26,31 @@ The base class will serve as documentation. And it will point to examples of
 it being used.
 """
 
+import inspect
 import os
-import pkgutil
-import shutil
 import sys
 from collections import namedtuple
 
 from nuitka.__past__ import getMetaClassBase
+from nuitka.freezer.IncludedEntryPoints import makeDllEntryPoint
+from nuitka.ModuleRegistry import getModuleInclusionInfoByName
+from nuitka.Options import isStandaloneMode
 from nuitka.Tracing import plugins_logger
 from nuitka.utils.Execution import NuitkaCalledProcessError, check_output
-from nuitka.utils.FileOperations import makePath
+from nuitka.utils.FileOperations import copyFile, makePath
 from nuitka.utils.ModuleNames import ModuleName
-
-pre_modules = {}
-post_modules = {}
+from nuitka.utils.SharedLibraries import locateDLL, locateDLLsInDirectory
 
 warned_unused_plugins = set()
 
+# Trigger names for shared use.
+postload_trigger_name = "postLoad"
+preload_trigger_name = "preLoad"
+
+
+def makeTriggerModuleName(module_name, trigger_name):
+    return ModuleName(module_name + "-" + trigger_name)
+
 
 class NuitkaPluginBase(getMetaClassBase("Plugin")):
     """Nuitka base class for all plugins.
@@ -169,17 +177,6 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return ()
 
-    def considerFailedImportReferrals(self, module_name):
-        """Provide a dictionary of fallback imports for modules that failed to import.
-
-        Args:
-            module_name: name of module
-        Returns:
-            dict
-        """
-        # Virtual method, pylint: disable=no-self-use,unused-argument
-        return None
-
     def onModuleSourceCode(self, module_name, source_code):
         """Inspect or modify source code.
 
@@ -268,6 +265,42 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=unused-argument
         return None
 
+    @staticmethod
+    def createFakeModuleDependency(module):
+        """Create module to depend on.
+
+        Notes:
+            Called by @onModuleDiscovered.
+
+        Args:
+            module: the module object
+
+        Returns:
+            None (does not apply, default)
+            tuple (code, reason)
+            tuple (code, reason, flags)
+        """
+        # Virtual method, pylint: disable=unused-argument
+        return None
+
+    @staticmethod
+    def hasPreModuleLoadCode(module_name):
+        return (
+            getModuleInclusionInfoByName(
+                makeTriggerModuleName(module_name, preload_trigger_name)
+            )
+            is not None
+        )
+
+    @staticmethod
+    def hasPostModuleLoadCode(module_name):
+        return (
+            getModuleInclusionInfoByName(
+                makeTriggerModuleName(module_name, postload_trigger_name)
+            )
+            is not None
+        )
+
     def onModuleDiscovered(self, module):
         """Called with a module to be loaded.
 
@@ -291,7 +324,7 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         Args:
             module_filename: filename
             module_name: full module name
-            module_kind: one of "py", "shlib" (shared library)
+            module_kind: one of "py", "extension" (shared library)
         Returns:
             True or False
         """
@@ -323,7 +356,7 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         """
 
     @staticmethod
-    def locateModule(importing, module_name):
+    def locateModule(module_name):
         """Provide a filename / -path for a to-be-imported module.
 
         Args:
@@ -332,42 +365,60 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         Returns:
             filename for module
         """
-        from nuitka.importing import Importing
 
-        _module_package, module_filename, _finding = Importing.findModule(
-            importing=importing,
-            module_name=ModuleName(module_name),
-            parent_package=None,
-            level=-1,
-            warn=False,
+        from nuitka.importing.Importing import locateModule
+
+        _module_name, module_filename, _finding = locateModule(
+            module_name=ModuleName(module_name), parent_package=None, level=0
         )
 
         return module_filename
 
-    def locateModules(self, importing, module_name):
+    @staticmethod
+    def locateModules(module_name):
         """Provide a filename / -path for a to-be-imported module.
 
         Args:
-            importing: module object that asked for it (tracing only)
             module_name: (str or ModuleName) full name of module
-            warn: (bool) True if required module
         Returns:
             list of ModuleName
         """
-        module_path = self.locateModule(importing, module_name)
 
-        result = []
+        from nuitka.importing.Importing import locateModules
 
-        def _scanModules(path, prefix):
-            for module_info in pkgutil.walk_packages((path,), prefix=prefix + "."):
-                result.append(ModuleName(module_info[1]))
+        return locateModules(module_name)
 
-                if module_info[2]:
-                    _scanModules(module_info[1], module_name + module_info[1])
+    @classmethod
+    def locateDLL(cls, dll_name):
+        """Locate a DLL by name."""
+        return locateDLL(dll_name)
 
-        _scanModules(module_path, module_name)
+    @classmethod
+    def locateDLLsInDirectory(cls, directory):
+        """Locate all DLLs in a folder
 
-        return result
+        Returns:
+            list of (filename, filename_relative, dll_extension)
+        """
+        return locateDLLsInDirectory(directory)
+
+    @classmethod
+    def makeDllEntryPoint(cls, source_path, dest_path, package_name):
+        """Create an entry point, as expected to be provided by getExtraDlls."""
+        return makeDllEntryPoint(
+            source_path=source_path, dest_path=dest_path, package_name=package_name
+        )
+
+    def reportFileCount(self, module_name, count, section=None):
+        if count:
+            msg = "Found %d %s DLLs from %s%s installation." % (
+                count,
+                "file" if count < 2 else "files",
+                "" if not section else (" '%s' " % section),
+                module_name.asString(),
+            )
+
+            self.info(msg)
 
     def considerExtraDlls(self, dist_dir, module):
         """Provide a tuple of names of binaries to be included.
@@ -379,14 +430,11 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
             tuple
         """
         # TODO: This should no longer be here, as this API is obsolete, pylint: disable=unused-argument
-
         for included_entry_point in self.getExtraDlls(module):
-            # Copy to the dist directory, which normally should not be our task, but is for now.
+            # Copy to the dist directory, which normally should not be a plugin task, but is for now.
             makePath(os.path.dirname(included_entry_point.dest_path))
 
-            shutil.copyfile(
-                included_entry_point.source_path, included_entry_point.dest_path
-            )
+            copyFile(included_entry_point.source_path, included_entry_point.dest_path)
 
             yield included_entry_point
 
@@ -402,6 +450,21 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return ()
 
+    def onCopiedDLL(self, dll_filename):
+        """Chance for a plugin to modify DLLs after copy, e.g. to compress it, remove attributes, etc.
+
+        Args:
+            dll_filename: the filename of the DLL
+
+        Notes:
+            Do not remove or add any files in this method, this will not work well, there
+            is e.g. getExtraDLLs API to add things. This is only for post processing as
+            described above.
+
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
     def getModuleSpecificDllPaths(self, module_name):
         """Provide a list of directories, where DLLs should be searched for this package (or module).
 
@@ -474,6 +537,18 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
 
+    def onBootstrapBinary(self, filename):
+        """Called after successfully creating a bootstrap binary, but without payload.
+
+        Args:
+            filename: the created bootstrap binary, will be modified later
+
+        Returns:
+            None
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
     def onFinalResult(self, filename):
         """Called after successfully finishing a compilation.
 
@@ -504,7 +579,7 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return False
 
-    def decideCompilation(self, module_name, source_ref):
+    def decideCompilation(self, module_name):
         """Decide whether to compile a module (or just use its bytecode).
 
         Notes:
@@ -514,10 +589,9 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
 
         Args:
             module_name: name of module
-            source_ref: ???
 
         Returns:
-            "compiled" or "bytecode" or None (default)
+            "compiled" or "bytecode" or None (no opinion, use by default)
         """
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
@@ -537,6 +611,16 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
         # Virtual method, pylint: disable=no-self-use
         return None
 
+    def getExtraIncludeDirectories(self):
+        """Decide which extra directories to use for C includes in compilation.
+
+        Returns:
+            List of directories or None by default
+        """
+
+        # Virtual method, pylint: disable=no-self-use
+        return None
+
     def getExtraCodeFiles(self):
         """Add extra code files to the compilation.
 
@@ -616,6 +700,8 @@ class NuitkaPluginBase(getMetaClassBase("Plugin")):
     _runtime_information_cache = {}
 
     def queryRuntimeInformationMultiple(self, info_name, setup_codes, values):
+        info_name = self.plugin_name.replace("-", "_") + "_" + info_name
+
         if info_name in self._runtime_information_cache:
             return self._runtime_information_cache[info_name]
 
@@ -699,25 +785,20 @@ except ImportError:
         plugins_logger.sysexit(cls.plugin_name + ": " + message)
 
 
-def isTriggerModule(module):
-    return module in pre_modules.values() or module in post_modules.values()
+import functools
 
 
-def replaceTriggerModule(old, new):
-    found = None
-    for key, value in pre_modules.items():
-        if value is old:
-            found = key
-            break
+def standalone_only(func):
+    """For plugins that have functionality that should be done in standalone mode only."""
 
-    if found is not None:
-        pre_modules[found] = new
+    @functools.wraps(func)
+    def wrapped(*args, **kwargs):
+        if isStandaloneMode():
+            return func(*args, **kwargs)
+        else:
+            if inspect.isgeneratorfunction(func):
+                return ()
+            else:
+                return None
 
-    found = None
-    for key, value in post_modules.items():
-        if value is old:
-            found = key
-            break
-
-    if found is not None:
-        post_modules[found] = new
+    return wrapped
