@@ -21,7 +21,7 @@ from nuitka import Options
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 
 
-class TensorflowPlugin(NuitkaPluginBase):
+class NuitkaPluginTensorflow(NuitkaPluginBase):
     """This class represents the main logic of the plugin.
 
     This is a plugin to ensure tensorflow scripts compile and work well in
@@ -62,6 +62,7 @@ class TensorflowPlugin(NuitkaPluginBase):
         """
         if module_name != "tensorflow":
             return source_code
+
         source_lines = source_code.splitlines()
         found_insert = False
         for i, l in enumerate(source_lines):
@@ -74,11 +75,11 @@ class TensorflowPlugin(NuitkaPluginBase):
         if found_insert is True:
             self.info("Patched 'running-from-pip' path magic.")
         else:
-            self.sysexit("Did not find path magic code." % self.plugin_name)
+            self.sysexit("Did not find 'running-from-pip' path magic code.")
 
         return "\n".join(source_lines)
 
-    def decideCompilation(self, module_name, source_ref):
+    def decideCompilation(self, module_name):
         """Include major packages as bytecode.
 
         Notes:
@@ -97,18 +98,16 @@ class TensorflowPlugin(NuitkaPluginBase):
             "matplotlib",
         ):
             return "bytecode"
-        else:
-            return "compiled"
 
 
-class TensorflowPluginDetector(NuitkaPluginBase):
+class NuitkaPluginDetectorTensorflow(NuitkaPluginBase):
     """Only used if plugin is NOT activated.
 
     Notes:
         We are given the chance to issue a warning if we think we may be required.
     """
 
-    detector_for = TensorflowPlugin
+    detector_for = NuitkaPluginTensorflow
 
     @classmethod
     def isRelevant(cls):
