@@ -51,6 +51,7 @@ from nuitka.utils.FileOperations import (
     openTextFile,
     removeDirectory,
 )
+from nuitka.utils.InstalledPythons import findInstalledPython
 from nuitka.utils.Jinja2 import getTemplate
 from nuitka.utils.Utils import getOS
 
@@ -80,11 +81,21 @@ def check_result(*popenargs, **kwargs):
         return True
 
 
+_start_dir = None
+
+
 def goMainDir():
+    global _start_dir  # singleton, pylint: disable=global-statement
+    _start_dir = os.getcwd()
+
     # Go its own directory, to have it easy with path knowledge.
     os.chdir(os.path.dirname(os.path.abspath(sys.modules["__main__"].__file__)))
 
 
+def getStartDir():
+    return _start_dir
+
+
 _python_version_str = None
 _python_version = None
 _python_arch = None
@@ -1192,6 +1203,25 @@ def scanDirectoryForTestCases(dirname, template_context=None):
         yield filename
 
 
+def scanDirectoryForTestCaseFolders(dirname):
+    filenames = os.listdir(dirname)
+
+    for filename in sorted(filenames):
+        filename = os.path.join(dirname, filename)
+        filename = os.path.relpath(filename)
+
+        if (
+            not os.path.isdir(filename)
+            or filename.endswith(".build")
+            or filename.endswith(".dist")
+        ):
+            continue
+
+        filename_main = getMainProgramFilename(filename)
+
+        yield filename, filename_main
+
+
 def setupCacheHashSalt(test_code_path):
     assert os.path.exists(test_code_path)
 
@@ -1625,8 +1655,12 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
         if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/gobject"):
             continue
 
-        # PyQt5 seems to do this, but won't use contents then.
+        # PyQt5 and PySide6 seems to do this, but won't use contents then.
         if loaded_filename in (
+            "/usr/lib/qt6/plugins",
+            "/usr/lib/qt6",
+            "/usr/lib64/qt6/plugins",
+            "/usr/lib64/qt6",
             "/usr/lib/qt5/plugins",
             "/usr/lib/qt5",
             "/usr/lib64/qt5/plugins",
@@ -1710,6 +1744,20 @@ def getMainProgramFilename(filename):
 
     test_logger.sysexit(
         """\
-Error, no file ends with 'Main.py' or 'Main' in %s, incomplete test case."""
+Error, no file ends with 'Main.py' or 'Main' in '%s', incomplete test case."""
         % (filename)
     )
+
+
+def getInstalledPythonVersion(python_version, must_exist):
+    result = findInstalledPython(
+        python_versions=(python_version,), module_name=None, module_version=None
+    )
+
+    if result is None and must_exist:
+        test_logger.sysexit(
+            "Error, cannot find required Python version %s installation."
+            % python_version
+        )
+
+    return result
