@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,12 +17,12 @@
 #
 """ Reformulation of contraction expressions.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
-from nuitka.__past__ import intern  # pylint: disable=I0021,redefined-builtin
+from nuitka.__past__ import intern
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
     StatementReleaseVariable,
@@ -77,7 +77,7 @@ from nuitka.nodes.YieldNodes import (
 from nuitka.PythonVersions import python_version
 
 from .ReformulationAssignmentStatements import buildAssignmentStatements
-from .ReformulationBooleanExpressions import buildAndNode
+from .ReformulationBooleanExpressions import makeAndNode
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .TreeHelpers import (
@@ -98,7 +98,7 @@ def _makeIteratorCreation(provider, qual, for_asyncgen, source_ref):
             source_ref=source_ref,
         )
 
-        if not for_asyncgen or python_version < 370:
+        if not for_asyncgen or python_version < 0x370:
             result = ExpressionYieldFromWaitable(
                 expression=result, source_ref=source_ref
             )
@@ -130,7 +130,7 @@ def _getStopIterationName(qual):
 
 def _buildPython2ListContraction(provider, node, source_ref):
     # The contraction nodes are reformulated to function bodies, with loops as
-    # described in the developer manual. They use a lot of temporary names,
+    # described in the Developer Manual. They use a lot of temporary names,
     # nested blocks, etc. and so a lot of variable names.
 
     # Note: The assign_provider is only to cover Python2 list contractions,
@@ -175,14 +175,16 @@ def _buildPython2ListContraction(provider, node, source_ref):
         source_ref=source_ref.atInternal(),
     )
 
-    function_body.setBody(makeStatementsSequenceFromStatement(statement=statement))
+    function_body.setChild(
+        "body", makeStatementsSequenceFromStatement(statement=statement)
+    )
 
     return function_body
 
 
 def buildListContractionNode(provider, node, source_ref):
     # List contractions are dealt with by general code.
-    if python_version < 300:
+    if python_version < 0x300:
         return _buildPython2ListContraction(
             provider=provider, node=node, source_ref=source_ref
         )
@@ -218,7 +220,7 @@ def buildDictContractionNode(provider, node, source_ref):
         node=node,
         name="<dictcontraction>",
         emit_class=StatementDictOperationSet
-        if python_version < 380
+        if python_version < 0x380
         else StatementDictOperationSetKeyValue,
         start_value={},
         source_ref=source_ref,
@@ -256,7 +258,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
     is_async = any(getattr(qual, "is_async", 0) for qual in node.generators)
 
     # Some of the newly allowed stuff in 3.7 fails to set the async flag.
-    if not is_async and python_version >= 370:
+    if not is_async and python_version >= 0x370:
         is_async = detectFunctionBodyKind(nodes=node.generators)[0] in (
             "Asyncgen",
             "Coroutine",
@@ -285,7 +287,8 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
         maker_class = ExpressionMakeGeneratorObject
 
-    function_body.setBody(
+    function_body.setChild(
+        "body",
         makeStatementsSequenceFromStatements(
             StatementAssignmentVariable(
                 variable=iter_tmp,
@@ -313,7 +316,7 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
                 ),
                 source_ref=source_ref,
             ),
-        )
+        ),
     )
 
     statements, release_statements = _buildContractionBodyNode(
@@ -342,14 +345,15 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
         ),
     )
 
-    code_body.setBody(
+    code_body.setChild(
+        "body",
         makeStatementsSequenceFromStatement(
             statement=StatementsFrameGenerator(
                 statements=mergeStatements(statements, False),
                 code_object=code_object,
                 source_ref=source_ref,
             )
-        )
+        ),
     )
 
     return function_body
@@ -400,7 +404,7 @@ def _buildContractionBodyNode(
             )
         )
 
-    if for_asyncgen and python_version >= 370 and node.generators[0].is_async:
+    if for_asyncgen and python_version >= 0x370 and node.generators[0].is_async:
         statements.append(
             StatementAssignmentVariable(
                 variable=iter_tmp,
@@ -477,7 +481,7 @@ def _buildContractionBodyNode(
         if qual is node.generators[0]:
             iterator_ref = makeVariableRefNode(variable=iter_tmp, source_ref=source_ref)
 
-            if for_asyncgen and python_version >= 370:
+            if for_asyncgen and python_version >= 0x370:
                 iterator_ref = ExpressionYieldFromWaitable(
                     expression=iterator_ref, source_ref=source_ref
                 )
@@ -545,7 +549,7 @@ def _buildContractionBodyNode(
         if len(conditions) >= 1:
             loop_statements.append(
                 makeStatementConditional(
-                    condition=buildAndNode(values=conditions, source_ref=source_ref),
+                    condition=makeAndNode(values=conditions, source_ref=source_ref),
                     yes_branch=current_body,
                     no_branch=None,
                     source_ref=source_ref,
@@ -556,7 +560,7 @@ def _buildContractionBodyNode(
 
         nested_statements.append(
             StatementLoop(
-                body=StatementsSequence(
+                loop_body=StatementsSequence(
                     statements=mergeStatements(loop_statements), source_ref=source_ref
                 ),
                 source_ref=source_ref,
@@ -587,7 +591,7 @@ def _buildContractionBodyNode(
 
 def _buildContractionNode(provider, node, name, emit_class, start_value, source_ref):
     # The contraction nodes are reformulated to function bodies, with loops as
-    # described in the developer manual. They use a lot of temporary names,
+    # described in the Developer Manual. They use a lot of temporary names,
     # nested blocks, etc. and so a lot of variable names.
 
     function_body = ExpressionOutlineFunction(
@@ -643,7 +647,7 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
         ),
     )
 
-    if python_version < 300:
+    if python_version < 0x300:
         body = makeStatementsSequenceFromStatements(assign_iter_statement, statements)
     else:
         parent_module = provider.getParentModule()
@@ -672,6 +676,6 @@ def _buildContractionNode(provider, node, name, emit_class, start_value, source_
             ),
         )
 
-    function_body.setBody(body)
+    function_body.setChild("body", body)
 
     return function_body
