@@ -68,7 +68,7 @@ def displayError(dirname, filename):
 
 
 def main():
-    python_version = setup(needs_io_encoding=True)
+    python_version = setup(suite="onefile", needs_io_encoding=True)
 
     search_mode = createSearchMode()
 
@@ -99,12 +99,14 @@ def main():
             "cpython_cache",
             # To understand what is slow.
             "timing",
+            # The onefile can warn about zstandard not being installed.
+            "ignore_warnings",
         ]
 
-        if filename == "KeyboardInteruptTest.py":
+        if filename == "KeyboardInterruptTest.py":
             if getOS() == "Darwin":
                 reportSkip(
-                    "Exit code from KeybaordInterrupt on macOS is not yet good.",
+                    "Exit code from KeyboardInterrupt on macOS is not yet good.",
                     ".",
                     filename,
                 )
@@ -150,7 +152,7 @@ def main():
 
         binary_filename = filename[:-3] + (".exe" if os.name == "nt" else ".bin")
 
-        if filename == "KeyboardInteruptTest.py":
+        if filename == "KeyboardInterruptTest.py":
             continue
 
         # Then use "strace" on the result.
