@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-# nuitka-project: --enable-plugin=multiprocessing
+""" Test that checks that accelerated mode usage of multiprocessing works too. """
 
 from multiprocessing import freeze_support
 
